package com.ruoyi.system.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.vo.FcryVo;
import com.ruoyi.system.mapper.SysDictDataMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.system.mapper.FcryMapper;
import com.ruoyi.system.service.IFcryService;

import javax.servlet.http.HttpServletResponse;

/**
 * 方舱人员Service业务层处理
 * 
 * @author ruoyi
 * @date 2022-11-28
 */
@Service
public class FcryServiceImpl implements IFcryService 
{
    @Autowired
    private FcryMapper fcryMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private SysDictDataMapper sysDictDataMapper;

    /**
     * 查询方舱人员
     * 
     * @param id 方舱人员主键
     * @return 方舱人员
     */
    @Override
    public Fcry selectFcryById(Long id)
    {
        return fcryMapper.selectFcryById(id);
    }

    /**
     * 查询方舱人员列表
     * 
     * @param fcry 方舱人员
     * @return 方舱人员
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Fcry> selectFcryList(Fcry fcry)
    {
        return fcryMapper.selectFcryList(fcry);
    }
    /**
     * 查询导出方舱人员列表
     *
     * @param fcry 方舱人员
     * @return 方舱人员
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Fcry> selectFcryListExport(Fcry fcry)
    {
        return fcryMapper.selectFcryListExport(fcry);
    }

    /**
     * 新增方舱人员
     * 
     * @param fcry 方舱人员
     * @return 结果
     */
    @Override
    public int insertFcry(Fcry fcry)
    {
        if (CheckUtils.checkCard(fcry.getCardNo()))
            fcry.setCheckCard("正确");
        else
            throw new ServiceException("身份证格式不正确");
//        if (!CheckUtils.checkPhone(fcry.getPhone()))
//            throw new ServiceException("联系方式格式不正确");
        List<Gldry>list = fcryMapper.selectGldryByCard(fcry.getCardNo(),null);
        if (!list.isEmpty())
            throw new ServiceException("该身份证号已存在");
        fcry.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
        fcry.setCreateTime(DateUtils.getNowDate());
        return fcryMapper.insertFcry(fcry);
    }

    /**
     * 修改方舱人员
     * 
     * @param fcry 方舱人员
     * @return 结果
     */
    @Override
    public int updateFcry(Fcry fcry)
    {
        if (CheckUtils.checkCard(fcry.getCardNo()))
            fcry.setCheckCard("正确");
        else
            throw new ServiceException("身份证格式不正确");
//        if (!CheckUtils.checkPhone(fcry.getPhone()))
//            throw new ServiceException("联系方式格式不正确");
        List<Gldry>list = fcryMapper.selectGldryByCard(fcry.getCardNo(),fcry.getId());
        if (!list.isEmpty())
            throw new ServiceException("该身份证号已存在");
        fcry.setUpdateTime(DateUtils.getNowDate());
        return fcryMapper.updateFcry(fcry);
    }

    /**
     * 批量删除方舱人员
     * 
     * @param ids 需要删除的方舱人员主键
     * @return 结果
     */
    @Override
    public int deleteFcryByIds(Long[] ids)
    {
        return fcryMapper.deleteFcryByIds(ids);
    }

    /**
     * 删除方舱人员信息
     * 
     * @param id 方舱人员主键
     * @return 结果
     */
    @Override
    public int deleteFcryById(Long id)
    {
        return fcryMapper.deleteFcryById(id);
    }

    @Override
    public void importUser(List<FcryVo> residentialList, String operName, HttpServletResponse response) {
        if (StringUtils.isNull(residentialList) || residentialList.size() == 0)
        {
            throw new ServiceException("导入方舱人员管理数据不能为空！");
        }
        List<Fcry> trueList = new ArrayList<>();
        List<FcryVo> failureList = new ArrayList<>();
        List<Fcry> updateList = new ArrayList<>();
        for (FcryVo fcry : residentialList){
            if (StringUtils.isNotEmpty(fcry.getFcPlace())
                    && StringUtils.isNotEmpty(fcry.getJzName()) && StringUtils.isNotEmpty(fcry.getCardNo())
                    && StringUtils.isNotEmpty(fcry.getPhone()) && StringUtils.isNotEmpty(fcry.getArea())
                    && fcry.getZrTime() != null){
                String communityValue = "";
                String committeeValue = "";
                String streetValue = "";
                String areaValue = "";
                String fcValue = "";
                if (StringUtils.isNotEmpty(fcry.getCommunity())){ //如果小区不为空，去数据字典匹配value
                    List<SysDictData> community = DictUtils.getDictCache("community");
                    List<String> values = community.stream().filter(l->fcry.getCommunity().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        communityValue = values.get(0);
//                    else {
//                        fcry.setFailure("小区不存在");
//                        failureList.add(fcry);
//                        continue;
//                    }
                }
                if (StringUtils.isNotEmpty(fcry.getCommittee())){ //如果居委会不为空，去数据字典匹配value
                    List<SysDictData> committee = DictUtils.getDictCache("committee");
                    List<String> values = committee.stream().filter(l->fcry.getCommittee().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        committeeValue=values.get(0);
                    else {
                        committeeValue = "";
                        communityValue = "";
//                        fcry.setFailure("居委会不存在");
//                        failureList.add(fcry);
//                        continue;
                    }
                }
                if (StringUtils.isNotEmpty(fcry.getStreet())){ //如果街道镇不为空，去数据字典匹配value
                    List<SysDictData> street = DictUtils.getDictCache("street_town");
                    List<String> values = street.stream().filter(l->fcry.getStreet().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        streetValue=values.get(0);
                    else {
                        committeeValue = "";
                        communityValue = "";
                        streetValue = "";
//                        fcry.setFailure("街道不存在");
//                        failureList.add(fcry);
//                        continue;
                    }
                }
                if (StringUtils.isNotEmpty(fcry.getArea())){ //如果区不为空，去数据字典匹配value
                    List<SysDictData> area = DictUtils.getDictCache("residential_area");
                    List<String> values = area.stream().filter(l->fcry.getArea().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        areaValue =values.get(0);
                    else {
                        fcry.setFailure("居住地所在区不存在");
                        failureList.add(fcry);
                        continue;
                    }
                }
                if (StringUtils.isNotEmpty(fcry.getFcPlace())){ //如果方舱不为空，去数据字典匹配value
                    List<SysDictData> fcPlace = DictUtils.getDictCache("shelters");
                    List<String> values = fcPlace.stream().filter(l->fcry.getFcPlace().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        fcValue = values.get(0);
                    else {
                        fcry.setFailure("方舱不存在");
                        failureList.add(fcry);
                        continue;
                    }
                }
//                if ((StringUtils.isNotEmpty(fcry.getCommunity())&&StringUtils.isEmpty(fcry.getCommittee()))||
//                        (StringUtils.isNotEmpty(fcry.getCommittee())&&StringUtils.isEmpty(fcry.getStreet()))){
//                    fcry.setFailure("街道或居委会不存在");
//                    failureList.add(fcry);
//                    continue;
//                }
                List<Fcry> fcrys = fcryMapper.selectFcryByCard(fcry.getCardNo());
                if (!fcrys.isEmpty()){
                    fcry.setCommunity(communityValue);
                    fcry.setCommittee(committeeValue);
                    fcry.setStreet(streetValue);
                    fcry.setArea(areaValue);
                    fcry.setFcPlace(fcValue);
                    updateList.add(fcry);
                }
                else {
                    fcrys = residentialList.stream().filter(l -> fcry.getCardNo().equals(l.getCardNo())).collect(Collectors.toList());
                    if (fcrys.size()>1) {
                        fcry.setFailure("存在重复数据");
                        failureList.add(fcry);
                    } else {
                        if (!CheckUtils.checkPhone(fcry.getPhone())) {
                            fcry.setFailure("联系方式格式不正确");
                            failureList.add(fcry);
                        }else {
                            if (CheckUtils.checkCard(fcry.getCardNo())){
                                fcry.setCommunity(communityValue);
                                fcry.setCommittee(committeeValue);
                                fcry.setStreet(streetValue);
                                fcry.setArea(areaValue);
                                fcry.setFcPlace(fcValue);
                                fcry.setCheckCard("正确");
                                trueList.add(fcry);
                            }
                            else{
                                fcry.setCheckCard("不正确");
                                fcry.setFailure("身份证格式不正确");
                                failureList.add(fcry);
                            }
                        }
                    }
                }
            }else {
                if(StringUtils.isEmpty(fcry.getFcPlace()))
                    fcry.setFailure("方舱名称不能为空");
                else if ( StringUtils.isEmpty(fcry.getJzName()))
                    fcry.setFailure("居住人姓名不能为空");
                else if ( StringUtils.isEmpty(fcry.getCardNo()))
                    fcry.setFailure("身份证号码不能为空");
                else if (StringUtils.isEmpty(fcry.getPhone()))
                    fcry.setFailure("联系方式不能为空");
                else if (StringUtils.isEmpty(fcry.getArea()))
                    fcry.setFailure("居住地所在区不能为空");
                else if ( fcry.getZrTime() == null)
                    fcry.setFailure("转入时间不能为空");
                failureList.add(fcry);
            }
        }
        if (!updateList.isEmpty()) {
            for (Fcry fcry : updateList) {
                List<Fcry> fcries = new ArrayList<>();
                fcry.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
                fcry.setUpdateTime(DateUtils.getNowDate());
                fcries.add(fcry);
                fcryMapper.updateBatchFcry(fcries);
            }
        }
        if (!trueList.isEmpty()){
            for (Fcry fcry1 : trueList) {
                fcry1.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
                fcry1.setCreateTime(DateUtils.getNowDate());
            }
            fcryMapper.insertBatchFcry(trueList);
        }
        if (!failureList.isEmpty()) {
            ExcelUtil<FcryVo> util = new ExcelUtil<FcryVo>(FcryVo.class);
            util.exportExcel(response, failureList, "方舱人员管理数据");
        }
    }
}
