package com.ruoyi.system.service.impl;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.Zlqk;
import com.ruoyi.system.domain.vo.FkqkVo;
import com.ruoyi.system.mapper.SysDictDataMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.system.mapper.FkqkMapper;
import com.ruoyi.system.domain.Fkqk;
import com.ruoyi.system.service.IFkqkService;

import javax.servlet.http.HttpServletResponse;

/**
 * 小区封控情况Service业务层处理
 *
 * @author ruoyi
 * @date 2022-11-28
 */
@Service
public class FkqkServiceImpl implements IFkqkService
{
    @Autowired
    private FkqkMapper fkqkMapper;
    @Autowired
    private SysDictDataMapper sysDictDataMapper;
    /**
     * 查询小区封控情况
     *
     * @param id 小区封控情况主键
     * @return 小区封控情况
     */
    @Override
    public Fkqk selectFkqkById(Long id)
    {
        return fkqkMapper.selectFkqkById(id);
    }

    /**
     * 查询小区封控情况列表
     *
     * @param fkqk 小区封控情况
     * @return 小区封控情况
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Fkqk> selectFkqkList(Fkqk fkqk)
    {
        return fkqkMapper.selectFkqkList(fkqk);
    }
    /**
     * 查询导出小区封控情况列表
     *
     * @param fkqk 小区封控情况
     * @return 小区封控情况
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Fkqk> selectFkqkListExport(Fkqk fkqk)
    {
        return fkqkMapper.selectFkqkListExport(fkqk);
    }

    /**
     * 新增小区封控情况
     *
     * @param fkqk 小区封控情况
     * @return 结果
     */
    @Override
    public int insertFkqk(Fkqk fkqk)
    {
        String createBy = DataUtils.getValue(fkqk.getArea(), fkqk.getStreet(), fkqk.getCommittee());
        if(createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
        fkqk.setCreateBy(createBy);
        fkqk.setCreateTime(DateUtils.getNowDate());
        return fkqkMapper.insertFkqk(fkqk);
    }

    /**
     * 修改小区封控情况
     *
     * @param fkqk 小区封控情况
     * @return 结果
     */
    @Override
    public int updateFkqk(Fkqk fkqk)
    {
        String createBy = DataUtils.getValue(fkqk.getArea(), fkqk.getStreet(), fkqk.getCommittee());
        if(createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
        fkqk.setCreateBy(createBy);
        fkqk.setUpdateTime(DateUtils.getNowDate());
        return fkqkMapper.updateFkqk(fkqk);
    }

    /**
     * 批量删除小区封控情况
     *
     * @param ids 需要删除的小区封控情况主键
     * @return 结果
     */
    @Override
    public int deleteFkqkByIds(Long[] ids)
    {
        return fkqkMapper.deleteFkqkByIds(ids);
    }

    /**
     * 删除小区封控情况信息
     *
     * @param id 小区封控情况主键
     * @return 结果
     */
    @Override
    public int deleteFkqkById(Long id)
    {
        return fkqkMapper.deleteFkqkById(id);
    }

    @Override
    public void importUser(List<FkqkVo> residentialList, HttpServletResponse response) {
        if (StringUtils.isNull(residentialList) || residentialList.size() == 0)
        {
            throw new ServiceException("导入小区封控情况数据不能为空！");
        }
        List<Fkqk> trueList = new ArrayList<>();
        List<FkqkVo> failureList = new ArrayList<>();
        for (FkqkVo fkqk : residentialList){
            if (StringUtils.isNotEmpty(fkqk.getArea()) && StringUtils.isNotEmpty(fkqk.getFkHs())
                    && StringUtils.isNotEmpty(fkqk.getFkRs()) && fkqk.getFkTime() != null){
                String communityValue = "";
                String committeeValue = "";
                String streetValue = "";
                String areaValue = "";
                Date now;
                Calendar c = Calendar.getInstance();
                now = new Date(c.get(Calendar.YEAR), c.get(Calendar.MONTH) + 1, c.get(Calendar.DAY_OF_MONTH));
                String createBy = DataUtils.getValue(fkqk.getArea(), fkqk.getStreet(), fkqk.getCommittee());
//                if (createBy.equals("null"))
//                    throw new ServiceException("所选居住地、街道、社区不存在");
                if (StringUtils.isNotEmpty(fkqk.getCommunity())){ //如果小区不为空，去数据字典匹配value
                    List<SysDictData> community = DictUtils.getDictCache("community");
                    List<String> values = community.stream().filter(l->fkqk.getCommunity().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        communityValue = values.get(0);
//                    else {
//                        failureList.add(fkqk);
//                        continue;
//                    }
                }
                if (StringUtils.isNotEmpty(fkqk.getCommittee())){ //如果居委会不为空，去数据字典匹配value
                    List<SysDictData> committee = DictUtils.getDictCache("committee");
                    List<String> values = committee.stream().filter(l->fkqk.getCommittee().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        committeeValue=values.get(0);
                    else {
//                        failureList.add(fkqk);
//                        continue;
                        committeeValue = "";
                        communityValue = "";
                    }
                }
                if (StringUtils.isNotEmpty(fkqk.getStreet())){ //如果街道镇不为空，去数据字典匹配value
                    List<SysDictData> street = DictUtils.getDictCache("street_town");
                    List<String> values = street.stream().filter(l->fkqk.getStreet().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        streetValue=values.get(0);
                    else {
//                        failureList.add(fkqk);
//                        continue;
                        committeeValue = "";
                        communityValue = "";
                        streetValue = "";
                    }
                }
                if (StringUtils.isNotEmpty(fkqk.getArea())){ //如果区不为空，去数据字典匹配value
                    List<SysDictData> area = DictUtils.getDictCache("residential_area");
                    List<String> values = area.stream().filter(l->fkqk.getArea().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                    areaValue = values.get(0);
                    else {
                        fkqk.setFailure("居住地所在区不存在");
                        failureList.add(fkqk);
                        continue;
                    }
                }
//                if ((StringUtils.isNotEmpty(fkqk.getCommunity())&&StringUtils.isEmpty(fkqk.getCommittee()))||
//                        (StringUtils.isNotEmpty(fkqk.getCommittee())&&StringUtils.isEmpty(fkqk.getStreet()))){
//                    failureList.add(fkqk);
//                    continue;
//                }
//                if (fkqk.getCommunity().length() > 30){
//                    fkqk.setFailure("居住地所在区不存在");
//                    failureList.add(fkqk);
//                }else
                if (fkqk.getFloor().length() > 30){
                    fkqk.setFailure("楼号字符长度不能大于30");
                    failureList.add(fkqk);
                }else if (fkqk.getUnit().length()  > 4){
                    fkqk.setFailure("单元号字符长度不能大于4");
                    failureList.add(fkqk);
                }else if (fkqk.getFkTime().after(now)){
                    fkqk.setFailure("封控时间不能比当前时间晚");
                    failureList.add(fkqk);
                }else if (fkqk.getRemark().length() > 200){
                    fkqk.setFailure("备注字符长度不能大于200");
                    failureList.add(fkqk);
                }else {
                    fkqk.setCommunity(communityValue);
                    fkqk.setCommittee(committeeValue);
                    fkqk.setStreet(streetValue);
                    fkqk.setArea(areaValue);
//                    fkqk.setStreet(sysDictDataHave.get(0).getDictValue());
                    fkqk.setCreateBy(createBy);
                    fkqk.setCreateTime(DateUtils.getNowDate());
                    trueList.add(fkqk);
                }
            }else if (StringUtils.isEmpty(fkqk.getStreet()) && StringUtils.isEmpty(fkqk.getCommunity())
                    && StringUtils.isEmpty(fkqk.getFloor()) && StringUtils.isEmpty(fkqk.getUnit())
                    && fkqk.getFkTime() == null){
                continue;
            }else {
                if (StringUtils.isEmpty(fkqk.getArea()))
                    fkqk.setFailure("居住地所在区不能为空");
                else if (StringUtils.isEmpty(fkqk.getFkHs()))
                    fkqk.setFailure("涉及封控户数不能为空");
                else if (StringUtils.isEmpty(fkqk.getFkRs()))
                    fkqk.setFailure("涉及封控人数不能为空");
                else if (fkqk.getFkTime() == null)
                    fkqk.setFailure("封控时间不能为空");
                failureList.add(fkqk);
            }
        }
        if (!trueList.isEmpty()){
            fkqkMapper.insertBatchFkqk(trueList);
        }
        if (!failureList.isEmpty()) {
            ExcelUtil<FkqkVo> util = new ExcelUtil<FkqkVo>(FkqkVo.class);
            util.exportExcel(response, failureList, "小区封控情况管理数据");
        }
    }

    @Override
    public List<String> selectStreet() {
        List<String> stringList = fkqkMapper.selectStreet();
        return stringList;
    }
    @Override
    public List<String> selectArea() {
        List<String> stringList = fkqkMapper.selectArea();
        return stringList;
    }
    @Override
    public List<String> selectFc() {
        List<String> stringList = fkqkMapper.selectFc();
        return stringList;
    }
}
