package com.ruoyi.system.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.Hscj;
import com.ruoyi.system.domain.vo.GldryVo;
import com.ruoyi.system.mapper.SysDictDataMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.system.mapper.GldryMapper;
import com.ruoyi.system.domain.Gldry;
import com.ruoyi.system.service.IGldryService;

import javax.servlet.http.HttpServletResponse;

import static com.ruoyi.common.utils.SecurityUtils.getLoginUser;

/**
 * 隔离点人员Service业务层处理
 * 
 * @author ruoyi
 * @date 2022-11-28
 */
@Service
public class GldryServiceImpl implements IGldryService 
{
    @Autowired
    private GldryMapper gldryMapper;
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private SysDictDataMapper sysDictDataMapper;

    /**
     * 查询隔离点人员
     * 
     * @param id 隔离点人员主键
     * @return 隔离点人员
     */
    @Override
    public Gldry selectGldryById(Long id)
    {
        return gldryMapper.selectGldryById(id);
    }

    /**
     * 查询隔离点人员列表
     * 
     * @param gldry 隔离点人员
     * @return 隔离点人员
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Gldry> selectGldryList(Gldry gldry)
    {
        return gldryMapper.selectGldryList(gldry);
    }
    /**
     * 查询导出隔离点人员列表
     *
     * @param gldry 隔离点人员
     * @return 隔离点人员
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Gldry> selectGldryListExport(Gldry gldry)
    {
        return gldryMapper.selectGldryListExport(gldry);
    }

    /**
     * 新增隔离点人员
     * 
     * @param gldry 隔离点人员
     * @return 结果
     */
    @Override
    public int insertGldry(Gldry gldry)
    {
        if (CheckUtils.checkCard(gldry.getCardNo()))
            gldry.setCheckCard("正确");
        else
            throw new ServiceException("身份证格式不正确");
//        if (!CheckUtils.checkPhone(gldry.getPhone()))
//            throw new ServiceException("联系方式格式不正确");
        List<Gldry>list = gldryMapper.selectGldryByCard(gldry.getCardNo(),null);
        if (!list.isEmpty())
            throw new ServiceException("该身份证号已存在");
        gldry.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
        gldry.setCreateTime(DateUtils.getNowDate());
        return gldryMapper.insertGldry(gldry);
    }

    /**
     * 修改隔离点人员
     * 
     * @param gldry 隔离点人员
     * @return 结果
     */
    @Override
    public int updateGldry(Gldry gldry)
    {
        if (CheckUtils.checkCard(gldry.getCardNo()))
            gldry.setCheckCard("正确");
        else
            throw new ServiceException("身份证格式不正确");
//        if (!CheckUtils.checkPhone(gldry.getPhone()))
//            throw new ServiceException("联系方式格式不正确");
        List<Gldry>list = gldryMapper.selectGldryByCard(gldry.getCardNo(),gldry.getId());
        if (!list.isEmpty())
            throw new ServiceException("该身份证号已存在");
        gldry.setUpdateTime(DateUtils.getNowDate());
        return gldryMapper.updateGldry(gldry);
    }

    /**
     * 批量删除隔离点人员
     * 
     * @param ids 需要删除的隔离点人员主键
     * @return 结果
     */
    @Override
    public int deleteGldryByIds(Long[] ids)
    {
        return gldryMapper.deleteGldryByIds(ids);
    }

    /**
     * 删除隔离点人员信息
     * 
     * @param id 隔离点人员主键
     * @return 结果
     */
    @Override
    public int deleteGldryById(Long id)
    {
        return gldryMapper.deleteGldryById(id);
    }
    @Override
    public void importList(List<GldryVo> list, String operName, HttpServletResponse response){
        if (StringUtils.isNull(list) || list.size() == 0)
        {
            throw new ServiceException("导入数据不能为空！");
        }
        List<Gldry> trueList = new ArrayList<>();
        List<GldryVo> failureList = new ArrayList<>();


        for (GldryVo gldry:list){
            // 校验单张Excel是否有多个重复的身份证号
            if (StringUtils.isNotEmpty(gldry.getCardNo())){
                int isExist = 0;
                for (Gldry gldryTest : list){
                    if (gldry.getCardNo().equals(gldryTest.getCardNo())){
                        isExist++;
                    }
                }
                if (isExist > 1){
                    gldry.setFailure("身份证号重复");
                    failureList.add(gldry);
                    continue;
                }
            }
            if (StringUtils.isNotEmpty(gldry.getGlPlace())
                    && StringUtils.isNotEmpty(gldry.getRoom())
                    && StringUtils.isNotEmpty(gldry.getJzName())
                    && StringUtils.isNotEmpty(gldry.getCardNo())
                    && StringUtils.isNotEmpty(gldry.getPhone())
                    && StringUtils.isNotEmpty(gldry.getArea())
                    && gldry.getZrTime()!=null
            ){
//                Gldry hldryQuerry = new Gldry();
//                hldryQuerry.setCardNo(gldry.getCardNo());
                List<Gldry> gldries = gldryMapper.selectGldryByCard(gldry.getCardNo(),null);
//
//                SysUser loginUser = getLoginUser().getUser();
//                String glPlace = loginUser.getNickName();
//                if (!gldry.getGlPlace().equals(glPlace)){
//                    failureList.add(gldry);
//                    continue;
//                }
                // 校验 隔离点名称
//                if (StringUtils.isNotEmpty(gldry.getGlPlace())){
//                    List<SysDictData> isolation_point = DictUtils.getDictCache("isolation_point");
//                    List<String> values = isolation_point.stream().filter(l->gldry.getGlPlace().equals(l.getDictLabel()))
//                            .map(SysDictData::getDictValue).collect(Collectors.toList());
//                    if (!values.isEmpty())
//                        gldry.setGlPlace(values.get(0));
//                    else {
//                        failureList.add(gldry);
//                        continue;
//                    }
//                }
                // 校验四级联动是否对应字典的值
                // 校验 所在区
//                if (StringUtils.isNotEmpty(gldry.getArea())){
//                    List<SysDictData> residential_area = DictUtils.getDictCache("residential_area");
//                    List<String> values = residential_area.stream().filter(l->gldry.getArea().equals(l.getDictLabel()))
//                            .map(SysDictData::getDictValue).collect(Collectors.toList());
//                    if (!values.isEmpty())
//                        gldry.setArea(values.get(0));
//                    else {
//                        failureList.add(gldry);
//                        continue;
//                    }
//                }
//                // 校验 街道/镇
//                if (StringUtils.isNotEmpty(gldry.getStreet())){
//                    List<SysDictData> street_town = DictUtils.getDictCache("street_town");
//                    List<String> values = street_town.stream().filter(l->gldry.getStreet().equals(l.getDictLabel()))
//                            .map(SysDictData::getDictValue).collect(Collectors.toList());
//                    if (!values.isEmpty())
//                        gldry.setStreet(values.get(0));
//                    else {
//                        failureList.add(gldry);
//                        continue;
//                    }
//                }
//                // 校验 居委会
//                if (StringUtils.isNotEmpty(gldry.getCommittee())){
//                    List<SysDictData> committee = DictUtils.getDictCache("committee");
//                    List<String> values = committee.stream().filter(l->gldry.getCommittee().equals(l.getDictLabel()))
//                            .map(SysDictData::getDictValue).collect(Collectors.toList());
//                    if (!values.isEmpty())
//                        gldry.setCommittee(values.get(0));
//                    else {
//                        failureList.add(gldry);
//                        continue;
//                    }
//                }
//                // 校验 小区/村名称
//                if (StringUtils.isNotEmpty(gldry.getCommunity())){
//                    List<SysDictData> community = DictUtils.getDictCache("community");
//                    List<String> values = community.stream().filter(l->gldry.getCommunity().equals(l.getDictLabel()))
//                            .map(SysDictData::getDictValue).collect(Collectors.toList());
//                    if (!values.isEmpty())
//                        gldry.setCommunity(values.get(0));
//                    else {
//                        failureList.add(gldry);
//                        continue;
//                    }
//                }
                String community = "";
                String committee = "";
                String street = "";
                if (StringUtils.isNotEmpty(gldry.getCommunity())){ //如果小区不为空，去数据字典匹配value
                    List<SysDictData> communitys = DictUtils.getDictCache("community");
                    List<String> values = communitys.stream().filter(l->gldry.getCommunity().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        community=values.get(0);
//                    else {
//                        failureList.add(gldry);
//                        continue;
//                    }
                }
                if (StringUtils.isNotEmpty(gldry.getCommittee())){ //如果委员会不为空，去数据字典匹配value
                    List<SysDictData> communitys = DictUtils.getDictCache("committee");
                    List<String> values = communitys.stream().filter(l->gldry.getCommittee().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        committee=values.get(0);
                    else {
//                        failureList.add(gldry);
//                        continue;
                        committee = "";
                        community = "";
                    }
                }
                if (StringUtils.isNotEmpty(gldry.getStreet())){ //如果街道不为空，去数据字典匹配value
                    List<SysDictData> communitys = DictUtils.getDictCache("street_town");
                    List<String> values = communitys.stream().filter(l->gldry.getStreet().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        street=values.get(0);
                    else {
//                        failureList.add(gldry);
//                        continue;
                        committee = "";
                        community = "";
                        street = "";
                    }
                }
//                if ((StringUtils.isNotEmpty(community)&&StringUtils.isEmpty(committee))||
//                        (StringUtils.isNotEmpty(committee)&&StringUtils.isEmpty(street))){
//                    failureList.add(gldry);
//                    continue;
//                }
                if (!gldries.isEmpty()){
                    for (Gldry gldryUpdate:gldries){
                        gldry.setId(gldryUpdate.getId());
                        gldry.setCommunity(community);
                        gldry.setCommittee(committee);
                        gldry.setStreet(street);
                        gldryMapper.updateGldry(gldry);
                    }
                }else {
                    if (CheckUtils.checkCard(gldry.getCardNo())){
                        gldry.setCheckCard("正确");
//                        if (gldry.getIsYx().equals("是")){
//                            gldry.setIsYx("阳");
//                        }
//                        if (gldry.getIsYx().equals("否")){
//                            gldry.setIsYx("未阳");
//                        }
                        gldry.setCommunity(community);
                        gldry.setCommittee(committee);
                        gldry.setStreet(street);
                        trueList.add(gldry);
                    }else {
                        gldry.setCheckCard("不正确");
                        gldry.setFailure("身份证号不正确");
                        failureList.add(gldry);
                    }
                }
            }else {
                if(StringUtils.isEmpty(gldry.getGlPlace()))
                    gldry.setFailure("隔离点名称不能为空");
                else if(StringUtils.isEmpty(gldry.getRoom()))
                    gldry.setFailure("房间号不能为空");
                else if(StringUtils.isEmpty(gldry.getJzName()))
                    gldry.setFailure("居住人姓名不能为空");
                else if(StringUtils.isEmpty(gldry.getCardNo()))
                    gldry.setFailure("身份证号不能为空");
                else if(StringUtils.isEmpty(gldry.getPhone()))
                    gldry.setFailure("联系方式不能为空");
                else if(StringUtils.isEmpty(gldry.getArea()))
                    gldry.setFailure("居住地所在区不能为空");
                else if(gldry.getZrTime()==null)
                    gldry.setFailure("转入时间不能为空");
                failureList.add(gldry);
            }
        }

        if (!trueList.isEmpty()){
            for (Gldry gldry:trueList){
//                String createBy = DataUtils.getValue(gldry.getArea(),gldry.getStreet(),gldry.getCommittee());
//                if (createBy.equals("null")){
//                    throw new ServiceException("所选居住地、街道、社区不存在");
//                }
                gldry.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
                gldry.setCreateTime(DateUtils.getNowDate());
                gldryMapper.insertGldry(gldry);
            }
        }
        if (!failureList.isEmpty()){
            ExcelUtil<GldryVo> util = new ExcelUtil<GldryVo>(GldryVo.class);
            util.exportExcel(response,failureList,"隔离点人员错误数据");
        }
    }
}
