package com.ruoyi.system.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.vo.HscjErrorDto;
import com.ruoyi.system.mapper.SysDictDataMapper;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.system.mapper.HscjMapper;
import com.ruoyi.system.domain.Hscj;
import com.ruoyi.system.service.IHscjService;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletResponse;

/**
 * 核酸采集Service业务层处理
 *
 * @author ruoyi
 * @date 2022-11-28
 */
@Service
public class HscjServiceImpl implements IHscjService {
    @Autowired
    private HscjMapper hscjMapper;
    @Autowired
    private SysDictDataMapper sysDictDataMapper;

    /**
     * 查询核酸采集
     *
     * @param id 核酸采集主键
     * @return 核酸采集
     */
    @Override
    public Hscj selectHscjById(Long id) {
        return hscjMapper.selectHscjById(id);
    }

    /**
     * 查询核酸采集列表
     *
     * @param hscj 核酸采集
     * @return 核酸采集
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Hscj> selectHscjList(Hscj hscj) {
        return hscjMapper.selectHscjList(hscj);
    }

    /**
     * 查询导出核酸采集列表
     *
     * @param hscj 核酸采集
     * @return 核酸采集
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Hscj> selectHscjListExport(Hscj hscj) {
        return hscjMapper.selectHscjListExport(hscj);
    }

    /**
     * 新增核酸采集
     *
     * @param hscj 核酸采集
     * @return 结果
     */
    @Override
    public int insertHscj(Hscj hscj) {
        String createBy = DataUtils.getValue(hscj.getArea(), hscj.getStreet(), hscj.getCommittee());
        if (createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
        if (CheckUtils.checkCard(hscj.getCardNo()))
            hscj.setCheckCard("正确");
        else
            throw new ServiceException("身份证格式不正确");
//        List<Hscj> list = hscjMapper.checkCard(hscj);
//        if (!list.isEmpty())
//            throw new ServiceException("该身份证号当天不能重复提交");
        hscj.setCreateBy(createBy);
        hscj.setCreateTime(DateUtils.getNowDate());
        return hscjMapper.insertHscj(hscj);
    }

    /**
     * 修改核酸采集
     *
     * @param hscj 核酸采集
     * @return 结果
     */
    @Override
    public int updateHscj(Hscj hscj) {
        String createBy = DataUtils.getValue(hscj.getArea(), hscj.getStreet(), hscj.getCommittee());
        if (createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
        hscj.setCreateBy(createBy);
        hscj.setUpdateTime(DateUtils.getNowDate());
        return hscjMapper.updateHscj(hscj);
    }

    /**
     * 批量删除核酸采集
     *
     * @param ids 需要删除的核酸采集主键
     * @return 结果
     */
    @Override
    public int deleteHscjByIds(Long[] ids) {
        return hscjMapper.deleteHscjByIds(ids);
    }

    /**
     * 删除核酸采集信息
     *
     * @param id 核酸采集主键
     * @return 结果
     */
    @Override
    public int deleteHscjById(Long id) {
        return hscjMapper.deleteHscjById(id);
    }

    @Override
    public void importList(List<HscjErrorDto> list, HttpServletResponse response) {
        if (StringUtils.isNull(list) || list.size() == 0) {
            throw new ServiceException("导入数据不能为空！");
        }
        List<HscjErrorDto> trueList = new ArrayList<>();
        List<HscjErrorDto> failureList = new ArrayList<>();
        List<String> error;
        String q = "";
        String jd = "";
        String jwh = "";
        for (HscjErrorDto hscj : list) {
            q = "";
            jd = "";
            jwh = "";
            error = new ArrayList<>();
            hscj.setProvince("河北省");
            hscj.setCity("石家庄市");
            if (StringUtils.isEmpty(hscj.getCardNo()))
                error.add("身份证号不能为空");
            if (StringUtils.isEmpty(hscj.getArea()))
                error.add("区不能为空");
            if (StringUtils.isEmpty(hscj.getBoxhao()))
                error.add("箱号不能为空");
            if (StringUtils.isEmpty(hscj.getCjgh()))
                error.add("采集管号不能为空");
            if (StringUtils.isEmpty(hscj.getUserName()))
                error.add("姓名不能为空");
            if (StringUtils.isEmpty(hscj.getSex()))
                error.add("性别不能为空");
            if (ObjectUtils.isEmpty(hscj.getAge()))
                error.add("年龄不能为空");
            if (ObjectUtils.isEmpty((hscj.getCjTime())))
                error.add("采集时间不能为空");
            if (StringUtils.isEmpty(hscj.getCjName()))
                error.add("采集人姓名不能为空");
            if (StringUtils.isEmpty(hscj.getCjPhone()))
                error.add("采集人电话不能为空");
            if (StringUtils.isEmpty(hscj.getBblx()))
                error.add("标本类型不能为空");
            if (CollectionUtils.isEmpty(error)) {
                if (hscj.getArea().equals("开发区"))
                    hscj.setArea("高新区");
                List<SysDictData> area = DictUtils.getDictCache("residential_area");
                List<String> areas = area.stream().filter(l -> hscj.getArea().equals(l.getDictLabel()))
                        .map(SysDictData::getDictValue).collect(Collectors.toList());
                if (!areas.isEmpty())
                    q = areas.get(0);
                else {
                    hscj.setError("区不存在");
                    failureList.add(hscj);
                    continue;
                }
                if (StringUtils.isEmpty(hscj.getRygx())) {
                    hscj.setRygx("本人");
                }
                List<HscjErrorDto> hsctList = list.stream().filter(l ->
                                hscj.getCardNo().equals(l.getCardNo()) && l.getCjgh() != null && hscj.getCjgh().equals(l.getCjgh()))
                        .collect(Collectors.toList());
                if (hsctList.size() > 1) {
                    hscj.setError("存在重复数据");
                    failureList.add(hscj);
                    continue;
                }
                //如果街道不为空，去数据字典匹配value
                if (StringUtils.isNotEmpty(hscj.getStreet())) {
                    List<SysDictData> communitys = DictUtils.getDictCache("street_town");
                    List<String> values = communitys.stream().filter(l -> hscj.getStreet().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        jd = values.get(0);
                    else {
                        hscj.setStreet(null);
                        hscj.setCommittee(null);
                        hscj.setCommunity(null);
                    }
                }
                //如果委员会不为空，去数据字典匹配value
                if (StringUtils.isNotEmpty(hscj.getCommittee())) {
                    List<SysDictData> communitys = DictUtils.getDictCache("committee");
                    List<String> values = communitys.stream().filter(l -> hscj.getCommittee().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        jwh = values.get(0);
                    else {
                        hscj.setCommittee(null);
                        hscj.setCommunity(null);
                    }
                }
                //如果小区不为空，去数据字典匹配value
                if (StringUtils.isNotEmpty(hscj.getCommunity())) {
                    List<SysDictData> communitys = DictUtils.getDictCache("community");
                    List<String> values = communitys.stream().filter(l -> hscj.getCommunity().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        hscj.setCommunity(values.get(0));
                    else {
                        hscj.setCommunity(null);
                    }
                }
                if (CheckUtils.checkCard(hscj.getCardNo())) {
                    hscj.setArea(q);
                    hscj.setStreet(jd);
                    hscj.setCommittee(jwh);
                    hscj.setCheckCard("正确");
                    trueList.add(hscj);
                } else {
                    hscj.setError("身份证号校验不正确");
                    failureList.add(hscj);
                    continue;
                }
            } else {
                hscj.setError(error.stream().collect(Collectors.joining(",")));
                failureList.add(hscj);
            }
        }
        if (!trueList.isEmpty()) {
            // 循环批量添加数据
            int toIndex = 500;
            for (int i = 0; i < trueList.size(); i += 500) {
                if (i + 500 > trueList.size()) {
                    toIndex = trueList.size() - i;
                }
                List<HscjErrorDto> list1 = trueList.subList(i, i + toIndex);
                for (HscjErrorDto hscj : list1) {
                    String createBy = DataUtils.getValue(hscj.getArea(), hscj.getStreet(), hscj.getCommittee());
                    hscj.setCreateBy(createBy);
                    hscj.setCreateTime(DateUtils.getNowDate());
                }
                hscjMapper.insertBatch(list1);
            }
        }
        if (!failureList.isEmpty()) {
            ExcelUtil<HscjErrorDto> util = new ExcelUtil<HscjErrorDto>(HscjErrorDto.class);
            util.exportExcel(response, failureList, "核算采集错误数据");
        }
    }

    @Override
    public List<Hscj> checkCard(Hscj hscj) {
        return hscjMapper.checkCard(hscj);
    }

}
