package com.ruoyi.system.service.impl;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.Fcry;
import com.ruoyi.system.domain.Gldry;
import com.ruoyi.system.domain.Yxry;
import com.ruoyi.system.domain.vo.JcglryVo;
import com.ruoyi.system.mapper.FcryMapper;
import com.ruoyi.system.mapper.GldryMapper;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.system.mapper.JcglryMapper;
import com.ruoyi.system.domain.Jcglry;
import com.ruoyi.system.service.IJcglryService;

import javax.servlet.http.HttpServletResponse;

/**
 * 解除隔离人员Service业务层处理
 * 
 * @author ruoyi
 * @date 2022-12-01
 */
@Service
public class JcglryServiceImpl implements IJcglryService 
{
    @Autowired
    private JcglryMapper jcglryMapper;

    @Autowired
    private FcryMapper fcryMapper;

    @Autowired
    private GldryMapper gldryMapper;

    /**
     * 查询解除隔离人员
     * 
     * @param id 解除隔离人员主键
     * @return 解除隔离人员
     */
    @Override
    public Jcglry selectJcglryById(Long id)
    {
        Jcglry jcglry = jcglryMapper.selectJcglryById(id);
        // 根据身份证号查询方舱人员信息
        List<Fcry> fcries = fcryMapper.selectFcryByCard(jcglry.getCardNo());
        // 根据身份证号查询隔离人员
        List<Gldry> gldries = gldryMapper.selectGldryByCard(jcglry.getCardNo(),null);
        Fcry fcry = null;
        Gldry gldry = null;
        if (!fcries.isEmpty())
            fcry = fcries.stream().max(Comparator.comparing(Fcry::getZrTime)).get();
        if (!gldries.isEmpty())
            gldry = gldries.stream().max(Comparator.comparing(Gldry::getZrTime)).get();
        //0=是,1=否
        if (fcry!=null && gldry!=null) {
            //相等则返回0，date1大返回1，否则返回-1。
            if (fcry.getZrTime().compareTo(gldry.getZrTime()) > 0){
                if (null!=fcry.getZcTime())
                    jcglry.setZyStatus("0");
                else
                    jcglry.setZyStatus("1");
            } else {
                if (null!=gldry.getZcTime())
                    jcglry.setZyStatus("0");
                else
                    jcglry.setZyStatus("1");
            }
        } else if (fcry!=null) {
            if (null!=fcry.getZcTime())
                jcglry.setZyStatus("0");
            else
                jcglry.setZyStatus("1");
        } else if (gldry!=null) {
            if (null!=gldry.getZcTime())
                jcglry.setZyStatus("0");
            else
                jcglry.setZyStatus("1");
        }
        return jcglry;
    }

    /**
     * 查询解除隔离人员列表
     * 
     * @param jcglry 解除隔离人员
     * @return 解除隔离人员
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Jcglry> selectJcglryList(Jcglry jcglry)
    {
        List<Jcglry> list = jcglryMapper.selectJcglryList(jcglry);
        for (Jcglry yx : list) {
            // 根据身份证号查询方舱人员信息
            List<Fcry> fcries = fcryMapper.selectFcryByCard(yx.getCardNo());
            // 根据身份证号查询隔离人员
            List<Gldry> gldries = gldryMapper.selectGldryByCard(yx.getCardNo(),null);
            Fcry fcry = null;
            Gldry gldry = null;
            if (!fcries.isEmpty())
                fcry = fcries.stream().max(Comparator.comparing(Fcry::getZrTime)).get();
            if (!gldries.isEmpty())
                gldry = gldries.stream().max(Comparator.comparing(Gldry::getZrTime)).get();
            //0=是,1=否
            if (fcry!=null && gldry!=null) {
                //相等则返回0，date1大返回1，否则返回-1。
                if (fcry.getZrTime().compareTo(gldry.getZrTime()) > 0){
                    if (null!=fcry.getZcTime())
                        yx.setZyStatus("0");
                    else
                        yx.setZyStatus("1");
                } else {
                    if (null!=gldry.getZcTime())
                        yx.setZyStatus("0");
                    else
                        yx.setZyStatus("1");
                }
            } else if (fcry!=null) {
                if (null!=fcry.getZcTime())
                    yx.setZyStatus("0");
                else
                    yx.setZyStatus("1");
            } else if (gldry!=null) {
                if (null!=gldry.getZcTime())
                    yx.setZyStatus("0");
                else
                    yx.setZyStatus("1");
            }
        }
        return list;
    }
    /**
     * 查询导出解除隔离人员列表
     *
     * @param jcglry 解除隔离人员
     * @return 解除隔离人员
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Jcglry> selectJcglryListExport(Jcglry jcglry)
    {
        List<Jcglry> list = jcglryMapper.selectJcglryListExport(jcglry);
        for (Jcglry yx : list) {
            // 根据身份证号查询方舱人员信息
            List<Fcry> fcries = fcryMapper.selectFcryByCard(yx.getCardNo());
            // 根据身份证号查询隔离人员
            List<Gldry> gldries = gldryMapper.selectGldryByCard(yx.getCardNo(),null);
            Fcry fcry = null;
            Gldry gldry = null;
            if (!fcries.isEmpty())
                fcry = fcries.stream().max(Comparator.comparing(Fcry::getZrTime)).get();
            if (!gldries.isEmpty())
                gldry = gldries.stream().max(Comparator.comparing(Gldry::getZrTime)).get();
            //0=是,1=否
            if (fcry!=null && gldry!=null) {
                //相等则返回0，date1大返回1，否则返回-1。
                if (fcry.getZrTime().compareTo(gldry.getZrTime()) > 0){
                    if (null!=fcry.getZcTime())
                        yx.setZyStatus("0");
                    else
                        yx.setZyStatus("1");
                } else {
                    if (null!=gldry.getZcTime())
                        yx.setZyStatus("0");
                    else
                        yx.setZyStatus("1");
                }
            } else if (fcry!=null) {
                if (null!=fcry.getZcTime())
                    yx.setZyStatus("0");
                else
                    yx.setZyStatus("1");
            } else if (gldry!=null) {
                if (null!=gldry.getZcTime())
                    yx.setZyStatus("0");
                else
                    yx.setZyStatus("1");
            }
        }
        return list;
    }

    /**
     * 新增解除隔离人员
     * 
     * @param jcglry 解除隔离人员
     * @return 结果
     */
    @Override
    public int insertJcglry(Jcglry jcglry)
    {
        jcglry.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
        jcglry.setCreateTime(DateUtils.getNowDate());
        return jcglryMapper.insertJcglry(jcglry);
    }

    /**
     * 修改解除隔离人员
     * 
     * @param jcglry 解除隔离人员
     * @return 结果
     */
    @Override
    public int updateJcglry(Jcglry jcglry)
    {
        jcglry.setUpdateTime(DateUtils.getNowDate());
        return jcglryMapper.updateJcglry(jcglry);
    }

    /**
     * 批量删除解除隔离人员
     * 
     * @param ids 需要删除的解除隔离人员主键
     * @return 结果
     */
    @Override
    public int deleteJcglryByIds(Long[] ids)
    {
        return jcglryMapper.deleteJcglryByIds(ids);
    }

    /**
     * 删除解除隔离人员信息
     * 
     * @param id 解除隔离人员主键
     * @return 结果
     */
    @Override
    public int deleteJcglryById(Long id)
    {
        return jcglryMapper.deleteJcglryById(id);
    }

    /**
     * 导入数据
     *
     * @return 结果
     */
    @Override
    public void importList(List<JcglryVo> list, HttpServletResponse response) {
        if (StringUtils.isNull(list) || list.size() == 0) {
            throw new ServiceException("导入数据不能为空！");
        }
        List<Jcglry> trueList = new ArrayList<>();
        List<JcglryVo> failureList = new ArrayList<>();
        List<Jcglry> updateList = new ArrayList<>();
        for (JcglryVo zlqk : list) {
            if (StringUtils.isNotEmpty(zlqk.getCardNo()) && StringUtils.isNotEmpty(zlqk.getIsArea())
                    && StringUtils.isNotEmpty(zlqk.getPhone()) && StringUtils.isNotEmpty(zlqk.getName())
                    && StringUtils.isNotEmpty(zlqk.getGlPlace()) && zlqk.getRzTime()!= null
                    && null!=zlqk.getJcTime() && StringUtils.isNotEmpty(zlqk.getRylb())) {
//                String community = "";
//                String committee = "";
                String street = "";
                if (StringUtils.isNotEmpty(zlqk.getStreet())){ //如果街道不为空，去数据字典匹配value
                    List<SysDictData> communitys = DictUtils.getDictCache("street_town");
                    List<String> values = communitys.stream().filter(l->zlqk.getStreet().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        street=values.get(0);
//                    else {
//                        failureList.add(zlqk);
//                        continue;
//                    }
                }
                if (StringUtils.isNotEmpty(zlqk.getGlPlace())){ //如果街道不为空，去数据字典匹配value
                    List<SysDictData> glPlace = DictUtils.getDictCache("isolation_point");
                    glPlace.addAll(DictUtils.getDictCache("shelters"));
                    List<String> values = glPlace.stream().filter(l->zlqk.getGlPlace().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        zlqk.setGlPlace(values.get(0));
                    else {
                        zlqk.setFailure("解除酒店不存在");
                        failureList.add(zlqk);
                        continue;
                    }
                }

//                if ((StringUtils.isNotEmpty(community)&&StringUtils.isEmpty(committee))||
//                        (StringUtils.isNotEmpty(committee)&&StringUtils.isEmpty(street))){
//                    failureList.add(zlqk);
//                    continue;
//                }
                //校验身份证是否重复
                List<Jcglry> zlqks = jcglryMapper.selectYxryByCard(zlqk.getCardNo());
                if (!zlqks.isEmpty()) {
//                    zlqk.setCommunity(community);
//                    zlqk.setCommittee(committee);
                    zlqk.setStreet(street);
                    updateList.add(zlqk);
                } else {
                    zlqks = list.stream().filter(l -> zlqk.getCardNo().equals(l.getCardNo())).collect(Collectors.toList());
                    if (zlqks.size() > 1) {
                        zlqk.setFailure("身份证号重复");
                        failureList.add(zlqk);
                    } else {
                        if (!CheckUtils.checkPhone(zlqk.getPhone())) {
                            zlqk.setFailure("联系方式不正确");
                            failureList.add(zlqk);
                        } else {
                            if (CheckUtils.checkCard(zlqk.getCardNo())){
//                                zlqk.setCheckCard("正确");
//                                zlqk.setCommunity(community);
//                                zlqk.setCommittee(committee);
                                zlqk.setStreet(street);
                                trueList.add(zlqk);
                            } else {
//                                zlqk.setCheckCard("不正确");
                                zlqk.setFailure("身份证号不正确");
                                failureList.add(zlqk);
                            }
                        }
                    }
                }
            } else if (StringUtils.isEmpty(zlqk.getCardNo()) && StringUtils.isEmpty(zlqk.getIsArea())
                    && StringUtils.isEmpty(zlqk.getPhone()) && StringUtils.isEmpty(zlqk.getName())
                    && StringUtils.isEmpty(zlqk.getGlPlace()) && zlqk.getRzTime()== null
                    && null==zlqk.getJcTime() && StringUtils.isEmpty(zlqk.getRylb())) {
                continue;
            } else {
                if (StringUtils.isEmpty(zlqk.getCardNo()))
                    zlqk.setFailure("身份证号不能为空");
                if (StringUtils.isEmpty(zlqk.getIsArea()))
                    zlqk.setFailure("区域类别不能为空");
                if (StringUtils.isEmpty(zlqk.getPhone()))
                    zlqk.setFailure("联系方式不能为空");
                if (StringUtils.isEmpty(zlqk.getName()))
                    zlqk.setFailure("解除人员姓名不能为空");
                if (StringUtils.isEmpty(zlqk.getGlPlace()))
                    zlqk.setFailure("解除酒店不能为空");
                if (zlqk.getRzTime()== null)
                    zlqk.setFailure("入住时间不能为空");
                if (null==zlqk.getJcTime())
                    zlqk.setFailure("解除时间不能为空");
                if (StringUtils.isEmpty(zlqk.getRylb()))
                    zlqk.setFailure("人员类别不能为空");
                failureList.add(zlqk);
            }
        }
        if (!trueList.isEmpty()) {
            for (Jcglry zlqk : trueList) {
                zlqk.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
                zlqk.setCreateTime(DateUtils.getNowDate());
            }
            jcglryMapper.insertBatchYxry(trueList);
        }
        if (!updateList.isEmpty()) {
            for (Jcglry zlqk : updateList) {
                zlqk.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
                zlqk.setUpdateTime(DateUtils.getNowDate());
            }
            jcglryMapper.updateBatchYxry(updateList);
        }
        if (!failureList.isEmpty()) {
            ExcelUtil<JcglryVo> util = new ExcelUtil<JcglryVo>(JcglryVo.class);
            util.exportExcel(response, failureList, "解除隔离人员数据");
//            return AjaxResult.error(111,"导出失败数据"+failureList.size()+"条");
        }
//            return AjaxResult.success("数据导入成功");
    }
}
