package com.ruoyi.system.service.impl;

import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.system.mapper.FcryMapper;
import com.ruoyi.system.mapper.GldryMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.utils.DictUtils;
import com.ruoyi.system.mapper.SysDictDataMapper;
import com.ruoyi.system.service.ISysDictDataService;

/**
 * 字典 业务层处理
 * 
 * @author ruoyi
 */
@Service
public class SysDictDataServiceImpl implements ISysDictDataService
{
    @Autowired
    private SysDictDataMapper dictDataMapper;
    @Autowired
    private FcryMapper fcryMapper;
    @Autowired
    private GldryMapper gldryMapper;

    /**
     * 根据条件分页查询字典数据
     * 
     * @param dictData 字典数据信息
     * @return 字典数据集合信息
     */
    @Override
    public List<SysDictData> selectDictDataList(SysDictData dictData)
    {
        return dictDataMapper.selectDictDataList(dictData);
    }

    /**
     * 根据字典类型和字典键值查询字典数据信息
     * 
     * @param dictType 字典类型
     * @param dictValue 字典键值
     * @return 字典标签
     */
    @Override
    public String selectDictLabel(String dictType, String dictValue)
    {
        return dictDataMapper.selectDictLabel(dictType, dictValue);
    }

    /**
     * 根据字典数据ID查询信息
     * 
     * @param dictCode 字典数据ID
     * @return 字典数据
     */
    @Override
    public SysDictData selectDictDataById(Long dictCode)
    {
        return dictDataMapper.selectDictDataById(dictCode);
    }

    /**
     * 批量删除字典数据信息
     * 
     * @param dictCodes 需要删除的字典数据ID
     */
    @Override
    public void deleteDictDataByIds(Long[] dictCodes)
    {
        for (Long dictCode : dictCodes)
        {
            SysDictData data = selectDictDataById(dictCode);
            dictDataMapper.deleteDictDataById(dictCode);
            List<SysDictData> dictDatas = dictDataMapper.selectDictDataByType(data.getDictType());
            DictUtils.setDictCache(data.getDictType(), dictDatas);
        }
    }

    /**
     * 新增保存字典数据信息
     * 
     * @param data 字典数据信息
     * @return 结果
     */
    @Override
    public int insertDictData(SysDictData data)
    {
        int row = dictDataMapper.insertDictData(data);
        if (row > 0)
        {
            List<SysDictData> dictDatas = dictDataMapper.selectDictDataByType(data.getDictType());
            DictUtils.setDictCache(data.getDictType(), dictDatas);
        }
        return row;
    }

    /**
     * 修改保存字典数据信息
     * 
     * @param data 字典数据信息
     * @return 结果
     */
    @Override
    public int updateDictData(SysDictData data)
    {
        if ("corresponding_user".equals(data.getDictType())){
            List<SysDictData> dictDatas = dictDataMapper.selectDictDataByType(data.getDictType());
            List<String> values = dictDatas.stream().filter(i -> "shelters".equals(i.getDictLabel()))
                    .map(SysDictData::getDictValue).collect(Collectors.toList());
            JSONObject sheltersJson = null;
            if (!values.isEmpty())
                sheltersJson = JSON.parseObject(values.get(0));
            List<String> isolations = dictDatas.stream().filter(i -> "isolation".equals(i.getDictLabel()))
                    .map(SysDictData::getDictValue).collect(Collectors.toList());
            JSONObject isolationJson = null;
            if (!isolations.isEmpty())
                isolationJson = JSON.parseObject(isolations.get(0));
            JSONObject dateValue = JSON.parseObject(data.getDictValue());
            Iterator<String> keys = dateValue.keySet().iterator();
            if ("shelters".equals(data.getDictLabel())) {
                while (keys.hasNext()) {
                    String key = keys.next();
                    if (!((String) dateValue.get(key)).equals(sheltersJson.get(key))) {
                        fcryMapper.updateFcryByCreateBy((String) dateValue.get(key),(String)sheltersJson.get(key));
                    }
                }
            }else {
                while (keys.hasNext()) {
                    String key = keys.next();
                    if (!((String) dateValue.get(key)).equals(isolationJson.get(key))) {
                        gldryMapper.updateGldryByCreateBy((String)dateValue.get(key),(String)isolationJson.get(key));
                    }
                }
            }
        }
        int row = dictDataMapper.updateDictData(data);
        if (row > 0)
        {
            List<SysDictData> dictDatas = dictDataMapper.selectDictDataByType(data.getDictType());
            DictUtils.setDictCache(data.getDictType(), dictDatas);
        }
        return row;
    }
}
