package com.ruoyi.system.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.Yxry;
import com.ruoyi.system.domain.vo.WzzryVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.system.mapper.WzzryMapper;
import com.ruoyi.system.domain.Wzzry;
import com.ruoyi.system.service.IWzzryService;

import javax.servlet.http.HttpServletResponse;

/**
 * 危重症人员Service业务层处理
 * 
 * @author ruoyi
 * @date 2022-12-02
 */
@Service
public class WzzryServiceImpl implements IWzzryService 
{
    @Autowired
    private WzzryMapper wzzryMapper;

    /**
     * 查询危重症人员
     * 
     * @param id 危重症人员主键
     * @return 危重症人员
     */
    @Override
    public Wzzry selectWzzryById(Long id)
    {
        return wzzryMapper.selectWzzryById(id);
    }

    /**
     * 查询危重症人员列表
     * 
     * @param wzzry 危重症人员
     * @return 危重症人员
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Wzzry> selectWzzryList(Wzzry wzzry)
    {
        return wzzryMapper.selectWzzryList(wzzry);
    }
    /**
     * 查询导出危重症人员列表
     *
     * @param wzzry 危重症人员
     * @return 危重症人员
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Wzzry> selectWzzryListExport(Wzzry wzzry)
    {
        return wzzryMapper.selectWzzryListExport(wzzry);
    }

    /**
     * 新增危重症人员
     * 
     * @param wzzry 危重症人员
     * @return 结果
     */
    @Override
    public int insertWzzry(Wzzry wzzry)
    {
        String createBy = DataUtils.getValue("1", wzzry.getStreet(), wzzry.getCommittee());
        if (createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
        List<Wzzry> list = wzzryMapper.selectWzzryByCard(wzzry.getCardNo());
        if (!list.isEmpty()){
            wzzry.setId(list.get(0).getId());
            wzzry.setCreateBy(createBy);
            return wzzryMapper.updateWzzry(wzzry);
        }
        wzzry.setCreateBy(createBy);
        wzzry.setCreateTime(DateUtils.getNowDate());
        return wzzryMapper.insertWzzry(wzzry);
    }

    /**
     * 修改危重症人员
     * 
     * @param wzzry 危重症人员
     * @return 结果
     */
    @Override
    public int updateWzzry(Wzzry wzzry)
    {
        String createBy = DataUtils.getValue("1", wzzry.getStreet(), wzzry.getCommittee());
        if (createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
        wzzry.setCreateBy(createBy);
        wzzry.setUpdateTime(DateUtils.getNowDate());
        return wzzryMapper.updateWzzry(wzzry);
    }

    /**
     * 批量删除危重症人员
     * 
     * @param ids 需要删除的危重症人员主键
     * @return 结果
     */
    @Override
    public int deleteWzzryByIds(Long[] ids)
    {
        return wzzryMapper.deleteWzzryByIds(ids);
    }

    /**
     * 删除危重症人员信息
     * 
     * @param id 危重症人员主键
     * @return 结果
     */
    @Override
    public int deleteWzzryById(Long id)
    {
        return wzzryMapper.deleteWzzryById(id);
    }

    /**
     * 导入数据
     *
     * @return 结果
     */
    @Override
    public void importList(List<WzzryVo> list, HttpServletResponse response) {
        if (StringUtils.isNull(list) || list.size() == 0) {
            throw new ServiceException("导入数据不能为空！");
        }
        List<Wzzry> trueList = new ArrayList<>();
        List<WzzryVo> failureList = new ArrayList<>();
        List<Wzzry> updateList = new ArrayList<>();
        for (WzzryVo zlqk : list) {
            if (StringUtils.isNotEmpty(zlqk.getCardNo()) && StringUtils.isNotEmpty(zlqk.getUserName())
                    && StringUtils.isNotEmpty(zlqk.getPhone())&& StringUtils.isNotEmpty(zlqk.getBasicDisease())
                    && StringUtils.isNotEmpty(zlqk.getHospital())&& StringUtils.isNotEmpty(zlqk.getIsCare())
                    && StringUtils.isNotEmpty(zlqk.getIsSelf()) && StringUtils.isNotEmpty(zlqk.getUserStyle())
                    && null != zlqk.getCohabitantsNumber()) {
                String community = "";
                String committee = "";
                String street = "";
                if (StringUtils.isNotEmpty(zlqk.getCommunity())) { //如果小区不为空，去数据字典匹配value
                    List<SysDictData> communitys = DictUtils.getDictCache("community");
                    List<String> values = communitys.stream().filter(l -> zlqk.getCommunity().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        community = values.get(0);
//                    else {
//                        failureList.add(zlqk);
//                        continue;
//                    }
                }
                if (StringUtils.isNotEmpty(zlqk.getCommittee())) { //如果委员会不为空，去数据字典匹配value
                    List<SysDictData> communitys = DictUtils.getDictCache("committee");
                    List<String> values = communitys.stream().filter(l -> zlqk.getCommittee().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        committee = values.get(0);
                    else {
//                        failureList.add(zlqk);
//                        continue;
                        community = "";
                        committee = "";
                    }
                }
                if (StringUtils.isNotEmpty(zlqk.getStreet())) { //如果街道不为空，去数据字典匹配value
                    List<SysDictData> communitys = DictUtils.getDictCache("street_town");
                    List<String> values = communitys.stream().filter(l -> zlqk.getStreet().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        street = values.get(0);
                    else {
//                        failureList.add(zlqk);
//                        continue;
                        community = "";
                        committee = "";
                        street = "";
                    }
                }
//                if ((StringUtils.isNotEmpty(community) && StringUtils.isEmpty(committee)) ||
//                        (StringUtils.isNotEmpty(committee) && StringUtils.isEmpty(street))) {
//                    failureList.add(zlqk);
//                    continue;
//                }
                //校验身份证是否重复
                List<Wzzry> zlqks = wzzryMapper.selectWzzryByCard(zlqk.getCardNo());
                if (!zlqks.isEmpty()) {
                    zlqk.setCommunity(community);
                    zlqk.setCommittee(committee);
                    zlqk.setStreet(street);
                    updateList.add(zlqk);
                } else {
                    zlqks = list.stream().filter(l -> zlqk.getCardNo().equals(l.getCardNo())).collect(Collectors.toList());
                    if (zlqks.size() > 1) {
                        zlqk.setFailure( "身份证号重复");
                        failureList.add(zlqk);
                    } else {
                        if (!CheckUtils.checkPhone(zlqk.getPhone())) {
                            zlqk.setFailure("联系方式不正确");
                            failureList.add(zlqk);
                        } else {
                            if (CheckUtils.checkCard(zlqk.getCardNo())) {
//                                zlqk.setCheckCard("正确");
                                zlqk.setCommunity(community);
                                zlqk.setCommittee(committee);
                                zlqk.setStreet(street);
                                trueList.add(zlqk);
                            } else {
//                                zlqk.setCheckCard("不正确");
                                zlqk.setFailure("身份证号不正确");
                                failureList.add(zlqk);
                            }
                        }
                    }
                }
            } else if (StringUtils.isEmpty(zlqk.getCardNo()) && StringUtils.isEmpty(zlqk.getUserName())
                    && StringUtils.isEmpty(zlqk.getPhone()) && StringUtils.isEmpty(zlqk.getBasicDisease())
                    && StringUtils.isEmpty(zlqk.getHospital())&& StringUtils.isEmpty(zlqk.getIsCare())
                    && StringUtils.isEmpty(zlqk.getIsSelf()) && StringUtils.isEmpty(zlqk.getUserStyle())
                    && null == zlqk.getCohabitantsNumber()) {
                continue;
            } else {
                if (StringUtils.isEmpty(zlqk.getCardNo()))
                    zlqk.setFailure("身份证号不能为空");
                else if (StringUtils.isEmpty(zlqk.getUserName()))
                    zlqk.setFailure("姓名不能为空");
                else if (StringUtils.isEmpty(zlqk.getPhone()))
                    zlqk.setFailure("联系方式不能为空");
                else if (StringUtils.isEmpty(zlqk.getBasicDisease()))
                    zlqk.setFailure("基础病情况不能为空");
                else if (StringUtils.isEmpty(zlqk.getHospital()))
                    zlqk.setFailure("经常就诊医院不能为空");
                else if (StringUtils.isEmpty(zlqk.getIsCare()))
                    zlqk.setFailure("是否有人照顾不能为空");
                else if (StringUtils.isEmpty(zlqk.getIsSelf()))
                    zlqk.setFailure("生活能否自理不能为空");
                else if (StringUtils.isEmpty(zlqk.getUserStyle()))
                    zlqk.setFailure("人员类型不能为空");
                else if (null == zlqk.getCohabitantsNumber())
                    zlqk.setFailure("同住人数不能为空");
                failureList.add(zlqk);
            }
        }
        if (!trueList.isEmpty()) {
            for (Wzzry zlqk : trueList) {
                zlqk.setCreateBy(DataUtils.getValue(null, zlqk.getStreet(), zlqk.getCommittee()));
                zlqk.setCreateTime(DateUtils.getNowDate());
            }
            wzzryMapper.insertBatch(trueList);
        }
        if (!updateList.isEmpty()) {
            for (Wzzry zlqk : updateList) {
                zlqk.setCreateBy(DataUtils.getValue(null, zlqk.getStreet(), zlqk.getCommittee()));
                zlqk.setUpdateTime(DateUtils.getNowDate());
            }
            wzzryMapper.updateBatch(updateList);
        }
        if (!failureList.isEmpty()) {
            ExcelUtil<WzzryVo> util = new ExcelUtil<WzzryVo>(WzzryVo.class);
            util.exportExcel(response, failureList, "阳危重症人员数据");
//            return AjaxResult.error(111,"导出失败数据"+failureList.size()+"条");
        }
//            return AjaxResult.success("数据导入成功");
    }
}
