package com.ruoyi.system.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.vo.YcsbVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.system.mapper.YcsbMapper;
import com.ruoyi.system.domain.Ycsb;
import com.ruoyi.system.service.IYcsbService;

import javax.servlet.http.HttpServletResponse;

/**
 * 抗原异常上报Service业务层处理
 *
 * @author ruoyi
 * @date 2022-11-28
 */
@Service
public class YcsbServiceImpl implements IYcsbService
{
    @Autowired
    private YcsbMapper ycsbMapper;

    /**
     * 查询抗原异常上报
     *
     * @param id 抗原异常上报主键
     * @return 抗原异常上报
     */
    @Override
    public Ycsb selectYcsbById(Long id)
    {
        return ycsbMapper.selectYcsbById(id);
    }

    /**
     * 查询抗原异常上报列表
     *
     * @param ycsb 抗原异常上报
     * @return 抗原异常上报
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Ycsb> selectYcsbList(Ycsb ycsb)
    {
        return ycsbMapper.selectYcsbList(ycsb);
    }
    /**
     * 查询导出抗原异常上报列表
     *
     * @param ycsb 抗原异常上报
     * @return 抗原异常上报
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Ycsb> selectYcsbListExport(Ycsb ycsb)
    {
        return ycsbMapper.selectYcsbListExport(ycsb);
    }

    /**
     * 新增抗原异常上报
     *
     * @param ycsb 抗原异常上报
     * @return 结果
     */
    @Override
    public int insertYcsb(Ycsb ycsb)
    {
        String createBy = DataUtils.getValue(ycsb.getArea(), ycsb.getStreet(), ycsb.getCommittee());
        if(createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
        if (CheckUtils.checkCard(ycsb.getCardNo()))
            ycsb.setCheckCard("正确");
        else
            throw new ServiceException("身份证格式不正确");
        List<Ycsb> list = ycsbMapper.selectYcsbByCard(ycsb.getCardNo(), null);
        if (!list.isEmpty())
            throw new ServiceException("身份证号不能重复");
        ycsb.setCreateBy(createBy);
        ycsb.setCreateTime(DateUtils.getNowDate());
        return ycsbMapper.insertYcsb(ycsb);
    }

    /**
     * 修改抗原异常上报
     *
     * @param ycsb 抗原异常上报
     * @return 结果
     */
    @Override
    public int updateYcsb(Ycsb ycsb)
    {
        String createBy = DataUtils.getValue(ycsb.getArea(), ycsb.getStreet(), ycsb.getCommittee());
        if(createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
        List<Ycsb> list = ycsbMapper.selectYcsbByCard(ycsb.getCardNo(),ycsb.getId());
        if (!list.isEmpty())
            throw new ServiceException("身份证号不能重复");
        ycsb.setCreateBy(createBy);
        ycsb.setUpdateTime(DateUtils.getNowDate());
        return ycsbMapper.updateYcsb(ycsb);
    }

    /**
     * 批量删除抗原异常上报
     *
     * @param ids 需要删除的抗原异常上报主键
     * @return 结果
     */
    @Override
    public int deleteYcsbByIds(Long[] ids)
    {
        return ycsbMapper.deleteYcsbByIds(ids);
    }

    /**
     * 删除抗原异常上报信息
     *
     * @param id 抗原异常上报主键
     * @return 结果
     */
    @Override
    public int deleteYcsbById(Long id)
    {
        return ycsbMapper.deleteYcsbById(id);
    }

    /**
     * 导入
     * @param list 导入的list数据
     * @param response
     */
    @Override
    public void importList(List<YcsbVo> list, HttpServletResponse response) {
        if (StringUtils.isNull(list) || list.size() == 0) {
            throw new ServiceException("导入数据不能为空！");
        }
        List<Ycsb> trueList = new ArrayList<>();
        List<YcsbVo> failureList = new ArrayList<>();
        for (YcsbVo ycsb : list) {
            if (StringUtils.isNotEmpty(ycsb.getUserName())
                    && StringUtils.isNotEmpty(ycsb.getCardNo())
                    && StringUtils.isNotEmpty(ycsb.getArea())
                    && null != ycsb.getSbDate()) {
                // 校验身份证号是否重复(同excel表验证)
                List<Ycsb> ycsbList = list.stream().filter(i -> ycsb.getCardNo().equals(i.getCardNo())).collect(Collectors.toList());
                if (ycsbList.size() > 1) {
                    ycsb.setFailure("身份证号重复");
                    failureList.add(ycsb);
                } else {
                    // 校验身份证格式
                    if (CheckUtils.checkCard(ycsb.getCardNo())) {
                        ycsb.setCheckCard("正确");
                        // 校验手机号格式
                        if (!CheckUtils.checkPhone(ycsb.getPhone())) {
                            ycsb.setFailure("联系方式不正确");
                            failureList.add(ycsb);
                        } else {
                            if (StringUtils.isNotEmpty(ycsb.getCommunity())) {
                                List<SysDictData> communties = DictUtils.getDictCache("community");
                                List<String> values = this.checkParam(communties, ycsb.getCommunity());
                                if (!values.isEmpty())
//                                    failureList.add(ycsb);
//                                    continue;
                                    ycsb.setCommunity(values.get(0));
//                                } else {
//                                    ycsb.setCommunity(values.get(0));
//                                }
                            }
                            if (StringUtils.isNotEmpty(ycsb.getCommittee())) {
                                List<SysDictData> commitees = DictUtils.getDictCache("committee");
                                List<String> values = this.checkParam(commitees, ycsb.getCommittee());
                                if (values.isEmpty()) {
                                    ycsb.setCommunity("");
                                    ycsb.setCommittee("");
//                                    failureList.add(ycsb);
//                                    continue;
                                } else {
                                    ycsb.setCommittee(values.get(0));
                                }
                            }
                            if (StringUtils.isNotEmpty(ycsb.getStreet())) {
                                List<SysDictData> streets = DictUtils.getDictCache("street_town");
                                List<String> values = this.checkParam(streets, ycsb.getStreet());
                                if (values.isEmpty()) {
                                    ycsb.setCommunity("");
                                    ycsb.setCommittee("");
                                    ycsb.setStreet("");
//                                    failureList.add(ycsb);
//                                    continue;
                                } else {
                                    ycsb.setStreet(values.get(0));
                                }
                            }
//                            if (StringUtils.isNotEmpty(ycsb.getCommittee()) && StringUtils.isEmpty(ycsb.getStreet())) {
//                                failureList.add(ycsb);
//                                continue;
//                            }

//                            if (StringUtils.isNotEmpty(ycsb.getCommunity()) && StringUtils.isEmpty(ycsb.getCommittee())) {
//                                failureList.add(ycsb);
//                                continue;
//                            }
                            trueList.add(ycsb);
                        }
                    } else {
                        ycsb.setCheckCard("不正确");
                        ycsb.setFailure("身份证号不正确");
                        failureList.add(ycsb);
                    }
                }
            } else {
                if (StringUtils.isEmpty(ycsb.getUserName()))
                    ycsb.setFailure("姓名不能为空");
                else if (StringUtils.isEmpty(ycsb.getCardNo()))
                    ycsb.setFailure("身份证号不能为空");
                else if (StringUtils.isEmpty(ycsb.getArea()))
                    ycsb.setFailure("居住地所在区不能为空");
                else if (null == ycsb.getSbDate())
                    ycsb.setFailure("时间不能为空");
                failureList.add(ycsb);
            }
        }
        if (!trueList.isEmpty()) {
            for (Ycsb ycsb : trueList) {
                // 校验身份证号是否重复(数据库验证)
                List<Ycsb> ycsbList = ycsbMapper.selectYcsbByCard(ycsb.getCardNo(),null);
                if ("是".equals(ycsb.getHasAcid())) {
                    ycsb.setHasAcid("0");
                } else if ("否".equals(ycsb.getHasAcid())) {
                    ycsb.setHasAcid("1");
                }
                if (StringUtils.isEmpty(ycsb.getKyResult())) {
                    ycsb.setKyResult("0");
                }
                String createBy = DataUtils.getValue(ycsb.getArea(), ycsb.getStreet(), ycsb.getCommittee());
                if ("null".equals(createBy)) {
                    throw new ServiceException("所选居住地、街道、社区不存在");
                }
                ycsb.setCreateBy(createBy);
                ycsb.setCreateTime(DateUtils.getNowDate());
                if (ycsbList.size() > 0) {
                    ycsb.setId(ycsbList.get(0).getId());
                    ycsbMapper.updateYcsb(ycsb);
                    continue;
                }
                ycsb.setIsHandle("0");
                ycsbMapper.insertYcsb(ycsb);
            }
        }
        if (!failureList.isEmpty()) {
            ExcelUtil<YcsbVo> util = new ExcelUtil<>(YcsbVo.class);
            util.exportExcel(response, failureList, "抗原异常上报错误数据");
        }
    }

    private List<String> checkParam(List<SysDictData> dicList, String str) {
        return dicList.stream().filter(i -> str.equals(i.getDictLabel()))
                .map(SysDictData::getDictValue).collect(Collectors.toList());
    }

    @Override
    public List<Ycsb> checkCard(Ycsb ycsb) {
        return ycsbMapper.selectYcsbByCard(ycsb.getCardNo(), ycsb.getId());
    }

}
