package com.ruoyi.system.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CheckUtils;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanValidators;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.Yxry;
import com.ruoyi.system.domain.vo.ZlqkVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.mapper.ZlqkMapper;
import com.ruoyi.system.domain.Zlqk;
import com.ruoyi.system.service.IZlqkService;

import javax.servlet.http.HttpServletResponse;

/**
 * 治疗情况Service业务层处理
 * 
 * @author ruoyi
 * @date 2022-11-27
 */
@Service
public class ZlqkServiceImpl implements IZlqkService 
{
    @Autowired
    private ZlqkMapper zlqkMapper;

    /**
     * 查询治疗情况
     * 
     * @param id 治疗情况主键
     * @return 治疗情况
     */
    @Override
    public Zlqk selectZlqkById(Long id)
    {
        return zlqkMapper.selectZlqkById(id);
    }

    /**
     * 查询治疗情况列表
     * 
     * @param zlqk 治疗情况
     * @return 治疗情况
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Zlqk> selectZlqkList(Zlqk zlqk)
    {
        return zlqkMapper.selectZlqkList(zlqk);
    }
    /**
     * 查询导出治疗情况列表
     *
     * @param zlqk 治疗情况
     * @return 治疗情况
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Zlqk> selectZlqkListExport(Zlqk zlqk)
    {
        return zlqkMapper.selectZlqkListExport(zlqk);
    }

    /**
     * 新增治疗情况
     * 
     * @param zlqk 治疗情况
     * @return 结果
     */
    @Override
    public int insertZlqk(Zlqk zlqk)
    {
        if (CheckUtils.checkCard(zlqk.getCardNo()))
            zlqk.setCheckCard("正确");
        else
            throw new ServiceException("身份证格式不正确");
        zlqk.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
        zlqk.setCreateTime(DateUtils.getNowDate());
        return zlqkMapper.insertZlqk(zlqk);
    }

    /**
     * 修改治疗情况
     * 
     * @param zlqk 治疗情况
     * @return 结果
     */
    @Override
    public int updateZlqk(Zlqk zlqk)
    {
        zlqk.setUpdateTime(DateUtils.getNowDate());
        return zlqkMapper.updateZlqk(zlqk);
    }

    /**
     * 批量删除治疗情况
     * 
     * @param ids 需要删除的治疗情况主键
     * @return 结果
     */
    @Override
    public int deleteZlqkByIds(Long[] ids)
    {
        return zlqkMapper.deleteZlqkByIds(ids);
    }

    /**
     * 删除治疗情况信息
     * 
     * @param id 治疗情况主键
     * @return 结果
     */
    @Override
    public int deleteZlqkById(Long id)
    {
        return zlqkMapper.deleteZlqkById(id);
    }

    /**
     * 导入数据
     *
     * @return 结果
     */
    @Override
    public void importList(List<ZlqkVo> list, HttpServletResponse response)
    {
        if (StringUtils.isNull(list) || list.size() == 0)
        {
            throw new ServiceException("导入数据不能为空！");
        }
        List<Zlqk> trueList = new ArrayList<>();
        List<ZlqkVo> failureList = new ArrayList<>();
        List<Yxry> updateList = new ArrayList<>();
        for (ZlqkVo zlqk : list) {
            if (StringUtils.isNotEmpty(zlqk.getCardNo()) && StringUtils.isNotEmpty(zlqk.getUserName())
                    &&StringUtils.isNotEmpty(zlqk.getPhone()) && zlqk.getSbDate() != null
                    &&StringUtils.isNotEmpty(zlqk.getMedication())){
                //校验身份证是否重复
                List<Zlqk> zlqks = zlqkMapper.selectZlqkByCard(zlqk.getCardNo());
                if (!zlqks.isEmpty()) {
                    zlqk.setFailure("身份证号重复");
                    failureList.add(zlqk);
                }else {
                    zlqks = list.stream().filter(l -> zlqk.getCardNo().equals(l.getCardNo())).collect(Collectors.toList());
                    if (zlqks.size()>1) {
                        zlqk.setFailure("身份证号重复");
                        failureList.add(zlqk);
                    }else {
                        if (CheckUtils.checkCard(zlqk.getCardNo())) {
                            zlqk.setCheckCard("正确");
                            trueList.add(zlqk);
                        }else {
                            zlqk.setFailure("身份证号不正确");
                            zlqk.setCheckCard("不正确");
                            failureList.add(zlqk);
                        }
                    }
                }
            } else if (StringUtils.isEmpty(zlqk.getCardNo()) && StringUtils.isEmpty(zlqk.getUserName())
                    &&StringUtils.isEmpty(zlqk.getPhone()) && zlqk.getSbDate() == null && StringUtils.isEmpty(zlqk.getMedication())) {
                continue;
            } else {
                if (StringUtils.isEmpty(zlqk.getCardNo()))
                    zlqk.setFailure("身份证号不能为空");
                else if (StringUtils.isEmpty(zlqk.getUserName()))
                    zlqk.setFailure("姓名不能为空");
                else if (StringUtils.isEmpty(zlqk.getPhone()))
                    zlqk.setFailure("联系方式不能为空");
                else if (zlqk.getSbDate() == null)
                    zlqk.setFailure("治疗日期不能为空");
                else if (StringUtils.isEmpty(zlqk.getMedication()))
                    zlqk.setFailure("今日用药情况不能为空");
                failureList.add(zlqk);
            }
        }
        if (!trueList.isEmpty()){
            for (Zlqk zlqk : trueList) {
                zlqk.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
                zlqk.setCreateTime(DateUtils.getNowDate());
            }
            zlqkMapper.insertBatchZlqk(trueList);
        }
        if (!failureList.isEmpty()) {
            ExcelUtil<ZlqkVo> util = new ExcelUtil<ZlqkVo>(ZlqkVo.class);
            util.exportExcel(response, failureList, "治疗情况数据");
//            return AjaxResult.error(111,"导出失败数据"+failureList.size()+"条");
        }
//            return AjaxResult.success("数据导入成功");
    }
}
